---
title: C++ Style Guide
summary: The most important C++ style rules for UNICADO.
authors:
    - Sebastian Oberschwendtner
date: 2023-09-12
---
# UNICADO C++ Style Guide

!!! tip
    Consistency in code style is the most important thing!

---

## Header Files
### Inline Functions
- Define inline functions only if they are small (~ 10 lines or less).

### Function Parameter Ordering
- The sequence of parameters of a function is: first input parameters, then output parameters.
- Input parameters are passed either as value or const reference.
- Output or input/output parameters are passed as pointers.

:fontawesome-solid-arrow-right: In general, try to use **value semantics** instead of reference/pointer semantics.

---

## Classes
### Doing Work in Constructors
- No errors may be thrown in the constructor.
- Short initialization in the constructor is ok.
- Transfer complex initialization with larger calculation in initialization method (e.g. `Init()`).
- Do not call `init()` in the constructor, but after constructor call!

### Inheritance
- Destructor must always be virtual if it should be a base class for other classes.

### Declaration Order
- Declaration sequence: public, protected, private. In every section:
     - Typedefs and Enums
- Constants (static const data members)
- Constructors
- Destructor
- methods, incl. static methods
- Data Members (except static const data members)
- for constants: first const then data type, e.g.: `const int i = 1;`

### Write Short Functions
- Prefer small and focused functions.
- For more than 40 lines: split the function

### Ownership and Smart Pointers
- Always use Smart Pointers instead of normal pointers (raw pointers), since Smart Pointers free up storage space at a suitable location.
- Do not use `auto_ptr` (obsolete and not container-compatible) but `unique_ptr`.
- Use `shared_ptr` only if several pointers to the same object are needed, because more memory is used for the reference counting.

---

## Other C++ Features
### Preincrement and Predecrement
- For primitive data types, post-increment `i++` is the same as pre-increment `++i`.
- Pre-increment is faster for iterators because no copy is created.
- For iterators: use pre-increment

### auto
- Only use `auto` when the type is still clear to the reader.
- Example:

```cpp
/* static_cast holds the type clearly visible */
auto value = std::static_cast<int>(other_value);
```
- Use `auto` for **trailing return types** of functions, because we are doing **Modern C++** 😉 :

```cpp
auto some_function(int input) -> double
{
    /* do stuff */
    return value;
}
```
- Do __not__ use `auto` when the type is not clearly visible.
- Don't:

```cpp
auto value = get_value(); // type ist not readable
```

### Comparison of variables of type double
- In order to avoid a compiler warning if floating values are compared (== or !=) use the function `fabs(a-b) < ACCURACY_LOW / _MEDIUM / _HIGH`. **LOW**, **MEDIUM** and **HIGH** correspond to accuracies of **10e-4**, **10e-7** and **10e-10**, respectively. Choose the accuracy as necessary for each case. Alternatively, you can also use the function `accuracyCheck(a, b, ACCURACY_LOW / _MEDIUM / _HIGH)`.
- Accuracy constants can be found in the :fontawesome-solid-file: `constants.h` file within the unitConversion library.

---

## Naming
- Function names, variable names and file names should be descriptive: no simple letters such as "b" for span.
- Avoid abbreviations: Don't delete letters from words in the name to make it shorter e.g.: acft_dsgn instead of aircraft_design (usual and common abbreviations are fine in case of doubt)
- Lowercase file names, variable names and function names; capitalize new word
- Capitalize Structs, Typdefs and Enums
- Names should be as short as possible, but as long as necessary

---

## Comments
- The best code is self-explaining --> good type & variable names, etc.
- Documentation with Doxyblocks; see also "Documentation with Doxyblocks.pptx".
- You can use `\` and `@` to mark the __doxygen__ keywords.
- Generally everywhere, where the code is not directly understandable by itself, comments on readability and comprehensibility are necessary.
- But don't describe the code itself. Assume that the reading person knows C++.
- Do not use umlauts (ä -> ae etc.)

### File Comments
- Line comments for attributes in header: Declaration `/**<Comment*//`
- Each file should have a comment at the beginning describing its contents.
- Header file `.h`: Describes the classes declared in the file. [icon arrow-right} what are they for, how are they used?
- Source file `.cpp`: Contains more information about implementation details / algorithms.
- Implementation details and information about algorithms can also be mentioned in.h file, but then mention in `.cpp` file Location of documentation.
- The files should always start with a copyright statement.
- After the copyright, give some information about the file with:

```cpp
/**
 * \file name.extension
 * \author Name (Email)
 * \brief Short description of the content
 * \version major.minor.bugfix
 * \date yyyy-mm-dd
 */
```

### Class Comments
- Each class should have an accompanying comment describing what the class is for and how it is used
- Block comments for classes in source:

```cpp
/**
 * \class name
 * \brief short description of the class
 * \details detailed description of the class
 */
```

### Function Comments
- Function declaration: Describe what the function does and how it is used.
- Does the memory function occupy memory that must be released again by the caller?
- Can one of the arguments be a null pointer?
- Definition of function: Describe how the function performs its task.
- Give an overview of the individual steps.
- Explain the used "coding tricks".
- Explain why this implementation was chosen.
- Block comments for methods/constructors in Source:

```cpp
/**
 * \brief description of the function/method/constructor
 * \details detailed description of the method
 * \param name Description of the transfer parameter
 * \return name Description of the return value
 */
```

- Additional comments in the code only via `//`.
- Instructions for creating documentation with Doxyblocks can be found in the wiki.

### TODO Comments
- Use **TODO** comments to mark temporary code, temporary solutions or parts in need of improvement.
- Example for formatting: `// TODO(Name#_#MantisID:_) Comment`
- A **TODO** is not an obligation that the specified person will solve the problem.
- Specify an approximately date/time frame, if necessary, in which the problem is to be solved.

---

## Formatting
- General (ANSI) style with 4 spaces as indentation.

### Line Length
- Each line of code max. 120 characters’ long.
- Exception: Comment line with example or URL.

### Spaces vs. Tabs
- Use only spaces, 4 spaces for an indentation.
- IDE can be set to output spaces at "TAB".

:fontawesome-solid-arrow-right: For a more detailed description of settings see Wiki article on [Linter Setup]().

== Function Declarations and Definitions ==
- Return type and function name in one line.
- Parameter in the same line if it does not match: Parameters in several lines
- No space between function name and opening parenthesis.
- No space between parenthesis and parameter.
- Example:

```cpp
auto ClassName::functionName(Type par_name1, Type par_name2) -> return_type
{
    DoSomething(); // 2 space indent
}
```

### Horizontal Whitespace
- No spaces at end of line.
- For operators (=, <, >, +, -, ...): space before and after: `int a = 0;` (not `int a=0;`)
- Sometimes spaces can be removed by factors: `v = w*x + y/z;`
- In loops and conditions: Spaces before the beginning of parentheses and after semicolons; no additional spaces in parentheses.
- Can be automated in the IDE under the button Source "Formatter Insert Spaces around operators".

### Vertical Whitespace
- Minimize empty lines!
- Basic rule: The more code fits on the screen, the easier it is to follow the program flow.
- Blank lines at the beginning and end of functions can sometimes improve readability.

!!! tip
    This code style is based on the Google C++ Style Guide [GSG :octicons-link-external-16:](https://google.github.io/styleguide/cppguide.html) and contains some guidelines. For more information or more detailed specifications, read the Google Guide.

---

## Bibliography

**GSG**: Google, [Google C++ Style Guide :octicons-link-external-16:](https://google.github.io/styleguide/cppguide.html)

**ISOCPP**: IsoCpp, Bjarne Stroustrop & Herb Sutter, [C++ Core Guidelines :octicons-link-external-16:](http://isocpp.github.io/CppCoreGuidelines/CppCoreGuidelines)
